@@ -31,12 +31,17 @@ use warnings;
 require ZoneMinder::Base;
 require ZoneMinder::Object;
 require ZoneMinder::Storage;
+require ZoneMinder::Frame;
 require Date::Manip;
 require File::Find;
 require File::Path;
 require File::Copy;
 require File::Basename;
 require Number::Bytes::Human;
+require Date::Parse;
+require POSIX;
+use Date::Format qw(time2str);
+use Time::HiRes qw(gettimeofday tv_interval);
 
 #our @ISA = qw(ZoneMinder::Object);
 use parent qw(ZoneMinder::Object);
@@ -50,9 +55,9 @@ use parent qw(ZoneMinder::Object);
 use ZoneMinder::Config qw(:all);
 use ZoneMinder::Logger qw(:all);
 use ZoneMinder::Database qw(:all);
-require Date::Parse;
 
-use vars qw/ $table $primary_key %fields $serial @identified_by/;
+use vars qw/ $table $primary_key %fields $serial @identified_by %defaults $debug/;
+$debug = 0;
 $table = 'Events';
 @identified_by = ('Id');
 $serial = $primary_key = 'Id';
@@ -60,6 +65,7 @@ $serial = $primary_key = 'Id';
   Id
   MonitorId
   StorageId
+  SecondaryStorageId
   Name
   Cause
   StartTime
@@ -84,10 +90,21 @@ $serial = $primary_key = 'Id';
   StateId
   Orientation
   DiskSpace
+  SaveJPEGs
   Scheme
 );
+%defaults = (
+  Cause =>  q`'Unknown'`,
+  DefaultVideo  => q`''`,
+  TotScore => '0',
+  Archived  =>  '0',
+  Videoed  =>  '0',
+  Uploaded  =>  '0',
+  Emailed   =>  '0',
+  Messaged  =>  '0',
+  Executed  =>  '0',
+);
 
-use POSIX;
 
 sub Time {
   if ( @_ > 1 ) {
@@ -101,56 +118,10 @@ sub Time {
   return $_[0]{Time};
 }
 
-sub Name {
-  if ( @_ > 1 ) {
-    $_[0]{Name} = $_[1];
-  }
-  return $_[0]{Name};
-} # end sub Name
-
-sub find {
-  shift if $_[0] eq 'ZoneMinder::Event';
-  my %sql_filters = @_;
-
-  my $sql = 'SELECT * FROM Events';
-  my @sql_filters;
-  my @sql_values;
-
-  if ( exists $sql_filters{Name} ) {
-    push @sql_filters , ' Name = ? ';
-    push @sql_values, $sql_filters{Name};
-  }
-  if ( exists $sql_filters{Id} ) {
-    push @sql_filters , ' Id = ? ';
-    push @sql_values, $sql_filters{Id};
-  }
-
-  $sql .= ' WHERE ' . join(' AND ', @sql_filters ) if @sql_filters;
-  $sql .= ' LIMIT ' . $sql_filters{limit} if $sql_filters{limit};
-
-  my $sth = $ZoneMinder::Database::dbh->prepare_cached( $sql )
-    or Fatal( "Can't prepare '$sql': ".$ZoneMinder::Database::dbh->errstr() );
-  my $res = $sth->execute( @sql_values )
-    or Fatal( "Can't execute '$sql': ".$sth->errstr() );
-
-  my @results;
-
-  while( my $db_filter = $sth->fetchrow_hashref() ) {
-    my $filter = new ZoneMinder::Event( $$db_filter{Id}, $db_filter );
-    push @results, $filter;
-  } # end while
-  $sth->finish();
-  return @results;
-}
-
-sub find_one {
-  my @results = find(@_);
-  return $results[0] if @results;
-}
-
 sub getPath {
-  return Path( @_ );
+  return Path(@_);
 }
+
 sub Path {
   my $event = shift;
 
@@ -163,13 +134,18 @@ sub Path {
 
   if ( ! $$event{Path} ) {
     my $Storage = $event->Storage();
-    $$event{Path} = join('/', $Storage->Path(), $event->RelativePath() );
+		if ( (!$$event{StorageId}) or defined $Storage->Id() ) {
+			$$event{Path} = join('/', $Storage->Path(), $event->RelativePath());
+		} else {
+			Error("Storage area for $$event{StorageId} no longer exists in db.");
+		}
   }
   return $$event{Path};
 }
 
 sub Scheme {
   my $self = shift;
+
   $$self{Scheme} = shift if @_;
 
   if ( ! $$self{Scheme} ) {
@@ -186,16 +162,16 @@ sub Scheme {
 
 sub RelativePath {
   my $event = shift;
-  if ( @_ ) {
-    $$event{RelativePath} = $_[0];
-  }
+
+  $$event{RelativePath} = shift if @_;
 
   if ( ! $$event{RelativePath} ) {
     if ( $$event{Scheme} eq 'Deep' ) {
       if ( $event->Time() ) {
         $$event{RelativePath} = join('/',
             $event->{MonitorId},
-            strftime( '%y/%m/%d/%H/%M/%S',
+            POSIX::strftime(
+              '%y/%m/%d/%H/%M/%S',
               localtime($event->Time())
               ),
             );
@@ -207,7 +183,7 @@ sub RelativePath {
       if ( $event->Time() ) {
         $$event{RelativePath} = join('/',
             $event->{MonitorId},
-            strftime( '%Y-%m-%d', localtime($event->Time())),
+            POSIX::strftime('%Y-%m-%d', localtime($event->Time())),
             $event->{Id},
             );
       } else {
@@ -227,16 +203,16 @@ sub RelativePath {
 
 sub LinkPath {
   my $event = shift;
-  if ( @_ ) {
-    $$event{LinkPath} = $_[0];
-  }
+
+  $$event{LinkPath} = shift if @_;
 
   if ( ! $$event{LinkPath} ) {
     if ( $$event{Scheme} eq 'Deep' ) {
       if ( $event->Time() ) {
         $$event{LinkPath} = join('/',
             $event->{MonitorId},
-            strftime( '%y/%m/%d',
+            POSIX::strftime(
+              '%y/%m/%d',
               localtime($event->Time())
               ),
             '.'.$$event{Id}
@@ -288,8 +264,8 @@ sub createIdFile {
 sub GenerateVideo {
   my ( $self, $rate, $fps, $scale, $size, $overwrite, $format ) = @_;
 
-  my $event_path = $self->Path( );
-  chdir( $event_path );
+  my $event_path = $self->Path();
+  chdir($event_path);
   ( my $video_name = $self->{Name} ) =~ s/\s/_/g;
 
   my @file_parts;
@@ -315,10 +291,10 @@ sub GenerateVideo {
     $file_scale =~ s/_00//;
     $file_scale =~ s/(_\d+)0+$/$1/;
     $file_scale = 's'.$file_scale;
-    push( @file_parts, $file_scale );
+    push @file_parts, $file_scale;
   } elsif ( $size ) {
     my $file_size = 'S'.$size;
-    push( @file_parts, $file_size );
+    push @file_parts, $file_size;
   }
   my $video_file = join('-', $video_name, $file_parts[0], $file_parts[1] ).'.'.$format;
   if ( $overwrite || !-s $video_file ) {
@@ -394,7 +370,7 @@ sub delete {
 
   if ( $$event{Id} ) {
     # Need to have an event Id if we are to delete from the db.  
-    Info("Deleting event $event->{Id} from Monitor $event->{MonitorId} StartTime:$event->{StartTime}");
+    Info("Deleting event $event->{Id} from Monitor $event->{MonitorId} StartTime:$event->{StartTime} from ".$event->Path());
     $ZoneMinder::Database::dbh->ping();
 
     $ZoneMinder::Database::dbh->begin_work();
@@ -426,66 +402,119 @@ sub delete {
 sub delete_files {
   my $event = shift;
 
-  my $Storage = @_ ? $_[0] : new ZoneMinder::Storage($$event{StorageId});
-  my $storage_path = $Storage->Path();
+  foreach my $Storage (
+    @_ ? ($_[0]) : (
+      new ZoneMinder::Storage($$event{StorageId}),
+      ( $$event{SecondaryStorageId} ? new ZoneMinder::Storage($$event{SecondaryStorageId}) : () ),
+    ) ) {
+    my $storage_path = $Storage->Path();
 
-  if ( ! $storage_path ) {
-    Error("Empty storage path when deleting files for event $$event{Id} with storage id $$event{StorageId}");
-    return;
-  }
-
-  if ( ! $$event{MonitorId} ) {
-    Error("No monitor id assigned to event $$event{Id}");
-    return;
-  }
-  my $event_path = $event->RelativePath();
-  Debug("Deleting files for Event $$event{Id} from $storage_path/$event_path, scheme is $$event{Scheme}.");
-  if ( $event_path ) {
-    ( $storage_path ) = ( $storage_path =~ /^(.*)$/ ); # De-taint
-    ( $event_path ) = ( $event_path =~ /^(.*)$/ ); # De-taint
-
-    my $deleted = 0;
-    if ( $$Storage{Type} and ( $$Storage{Type} eq 's3fs' ) ) {
-      my ( $aws_id, $aws_secret, $aws_host, $aws_bucket ) = ( $$Storage{Url} =~ /^\s*([^:]+):([^@]+)@([^\/]*)\/(.+)\s*$/ );
-      eval {
-        require Net::Amazon::S3;
-        my $s3 = Net::Amazon::S3->new( {
-             aws_access_key_id     => $aws_id,
-             aws_secret_access_key => $aws_secret,
-             ( $aws_host ? ( host => $aws_host ) : () ),
-             });
-        my $bucket = $s3->bucket($aws_bucket);
-        if ( ! $bucket ) {
-          Error("S3 bucket $bucket not found.");
-          die;
-        }
-        if ( $bucket->delete_key($event_path) ) {
-          $deleted = 1;
-        } else {
-          Error('Failed to delete from S3:'.$s3->err . ': ' . $s3->errstr);
-        }
-      };
-      Error($@) if $@;
+    if ( !$storage_path ) {
+      Error("Empty storage path when deleting files for event $$event{Id} with storage id $$event{StorageId}");
+      return;
     }
-    if ( !$deleted ) {
-      my $command = "/bin/rm -rf $storage_path/$event_path";
-      ZoneMinder::General::executeShellCommand($command);
-    }
-  }
 
-  if ( $event->Scheme() eq 'Deep' ) {
-    my $link_path = $event->LinkPath();
-    Debug("Deleting link for Event $$event{Id} from $storage_path/$link_path.");
-    if ( $link_path ) {
-      ( $link_path ) = ( $link_path =~ /^(.*)$/ ); # De-taint
+    if ( !$$event{MonitorId} ) {
+      Error("No monitor id assigned to event $$event{Id}");
+      return;
+    }
+    my $event_path = $event->RelativePath();
+    Debug("Deleting files for Event $$event{Id} from $storage_path/$event_path, scheme is $$event{Scheme}.");
+    if ( $event_path ) {
+      ( $storage_path ) = ( $storage_path =~ /^(.*)$/ ); # De-taint
+      ( $event_path ) = ( $event_path =~ /^(.*)$/ ); # De-taint
+
+      my $deleted = 0;
+      if ( $$Storage{Type} and ( $$Storage{Type} eq 's3fs' ) ) {
+        my $url = $$Storage{Url};
+        $url =~ s/^(s3|s3fs):\/\///ig;
+        my ( $aws_id, $aws_secret, $aws_host, $aws_bucket, $subpath ) = ( $url =~ /^\s*([^:]+):([^@]+)@([^\/]*)\/([^\/]+)(\/.+)?\s*$/ );
+        Debug("S3 url parsed to id:$aws_id secret:$aws_secret host:$aws_host, bucket:$aws_bucket, subpath:$subpath\n from $url");
+        eval {
+          require Net::Amazon::S3;
+          my $s3 = Net::Amazon::S3->new( {
+              aws_access_key_id     => $aws_id,
+              aws_secret_access_key => $aws_secret,
+              ( $aws_host ? ( host => $aws_host ) : () ),
+              authorization_method => 'Net::Amazon::S3::Signature::V4',
+            });
+          my $bucket = $s3->bucket($aws_bucket);
+          if ( ! $bucket ) {
+            Error("S3 bucket $bucket not found.");
+            die;
+          }
+          if ( $bucket->delete_key($subpath.$event_path) ) {
+            $deleted = 1;
+          } else {
+            Error('Failed to delete from S3:'.$s3->err . ': ' . $s3->errstr);
+          }
+        };
+        Error($@) if $@;
+      } # end if s3fs
+      if ( !$deleted ) {
+        my $command = "/bin/rm -rf $storage_path/$event_path";
+        ZoneMinder::General::executeShellCommand($command);
+      }
+    } else {
+      Error('No event path in delete files. ' . $event->to_string());
+    } # end if event_path
+
+    if ( $event->Scheme() eq 'Deep' ) {
+      my $link_path = $event->LinkPath();
+      Debug("Deleting link for Event $$event{Id} from $storage_path/$link_path.");
+      if ( $link_path ) {
+        ( $link_path ) = ( $link_path =~ /^(.*)$/ ); # De-taint
         unlink($storage_path.'/'.$link_path) or Error("Unable to unlink '$storage_path/$link_path': $!");
-    }
-  }
+      }
+    } # end if Scheme eq Deep
+
+    # Now check for empty directories and delete them.
+    my @path_parts = split('/', $event_path);
+    pop @path_parts;
+    # Guaranteed the first part is the monitor id
+    Debug("Initial path_parts: @path_parts");
+    while ( @path_parts > 1 ) {
+      my $path = join('/', $storage_path, @path_parts);
+      my $dh;
+      if ( !opendir($dh, $path) ) {
+        Warning("Fail to open $path");
+        last;
+      }
+      my @dir =  readdir($dh);
+      closedir($dh);
+      if ( scalar(grep { $_ ne '.' and $_ ne '..' } @dir) == 0 ) {
+        Debug("Removing empty dir at $path");
+        if ( !rmdir $path ) {
+          Warning("Fail to rmdir $path: $!");
+          last;
+        }
+      } else {
+        Debug("Dir $path is not empty @dir");
+        last;
+      }
+      pop @path_parts;
+    } # end while path_parts
+
+  } # end foreach Storage
 } # end sub delete_files
 
+sub StorageId {
+  my $event = shift;
+  if ( @_ ) {
+    $$event{StorageId} = shift;
+    delete $$event{Storage};
+    $event->Path(undef);
+  }
+  return $$event{StorageId};
+}
+
 sub Storage {
   if ( @_ > 1 ) {
     $_[0]{Storage} = $_[1];
+    if ( $_[0]{Storage} ) {
+      $_[0]{StorageId} = $_[0]{Storage}->Id();
+      $_[0]->Path(undef);
+    }
   }
   if ( ! $_[0]{Storage} ) {
     $_[0]{Storage} = new ZoneMinder::Storage($_[0]{StorageId});
@@ -538,25 +567,29 @@ sub DiskSpace {
   return $_[0]{DiskSpace};
 }
 
-sub MoveTo {
+sub CopyTo {
   my ( $self, $NewStorage ) = @_;
 
   my $OldStorage = $self->Storage(undef);
   my ( $OldPath ) = ( $self->Path() =~ /^(.*)$/ ); # De-taint
   if ( ! -e $OldPath ) {
-    return "Old path $OldPath does not exist.";
+    return "Src path $OldPath does not exist.";
   }
   # First determine if we can move it to the dest.
   # We do this before bothering to lock the event
   my ( $NewPath ) = ( $NewStorage->Path() =~ /^(.*)$/ ); # De-taint
   if ( ! $$NewStorage{Id} ) {
-    return "New storage does not have an id.  Moving will not happen.";
+    return 'New storage does not have an id.  Moving will not happen.';
   } elsif ( $$NewStorage{Id} == $$self{StorageId} ) {
-    return "Event is already located at " . $NewPath;
+    return 'Event is already located at ' . $NewPath;
   } elsif ( !$NewPath ) {
     return "New path ($NewPath) is empty.";
-  } elsif ( ! -e $NewPath ) {
-    return "New path $NewPath does not exist.";
+  } elsif ( ($$NewStorage{Type} ne 's3fs' ) and ! -e $NewPath ) {
+    if ( ! mkdir($NewPath) ) {
+      return "New path $NewPath does not exist.";
+    }
+  } else {
+    Debug("$NewPath is good");
   }
 
   $ZoneMinder::Database::dbh->begin_work();
@@ -564,7 +597,7 @@ sub MoveTo {
   # data is reloaded, so need to check that the move hasn't already happened.
   if ( $$self{StorageId} == $$NewStorage{Id} ) {
     $ZoneMinder::Database::dbh->commit();
-    return "Event has already been moved by someone else.";
+    return 'Event has already been moved by someone else.';
   }
 
   if ( $$OldStorage{Id} != $$self{StorageId} ) {
@@ -572,76 +605,97 @@ sub MoveTo {
     return 'Old Storage path changed, Event has moved somewhere else.';
   }
 
-  $$self{Storage} = $NewStorage;
-  ( $NewPath ) = ( $self->Path(undef) =~ /^(.*)$/ ); # De-taint
+  Debug("Relative Path: " . $self->RelativePath());
+	$NewPath .= '/'.$self->RelativePath();
+	($NewPath) = ( $NewPath =~ /^(.*)$/ ); # De-taint
   if ( $NewPath eq $OldPath ) {
     $ZoneMinder::Database::dbh->commit();
     return "New path and old path are the same! $NewPath";
   }
-  Debug("Moving event $$self{Id} from $OldPath to $NewPath");
+  Debug("Copying event $$self{Id} from $OldPath to $NewPath");
 
   my $moved = 0;
 
   if ( $$NewStorage{Type} eq 's3fs' ) {
-    my ( $aws_id, $aws_secret, $aws_host, $aws_bucket ) = ( $$NewStorage{Url} =~ /^\s*([^:]+):([^@]+)@([^\/]*)\/(.+)\s*$/ );
-    eval {
-      require Net::Amazon::S3;
-      require File::Slurp;
-      my $s3 = Net::Amazon::S3->new( {
-          aws_access_key_id     => $aws_id,
-          aws_secret_access_key => $aws_secret,
-          ( $aws_host ? ( host => $aws_host ) : () ),
-          });
-      my $bucket = $s3->bucket($aws_bucket);
-      if ( ! $bucket ) {
-        Error("S3 bucket $bucket not found.");
-        die;
+    if ( $$NewStorage{Url} ) {
+      my $url = $$NewStorage{Url};
+      $url =~ s/^(s3|s3fs):\/\///ig;
+      my ( $aws_id, $aws_secret, $aws_host, $aws_bucket, $subpath ) = ( $url =~ /^\s*([^:]+):([^@]+)@([^\/]*)\/([^\/]+)(\/.+)?\s*$/ );
+      Debug("S3 url parsed to id:$aws_id secret:$aws_secret host:$aws_host, bucket:$aws_bucket, subpath:$subpath\n from $url");
+      if ( $aws_id and $aws_secret and $aws_host and $aws_bucket ) {
+        eval {
+          require Net::Amazon::S3;
+          require File::Slurp;
+          my $s3 = Net::Amazon::S3->new( {
+              aws_access_key_id     => $aws_id,
+              aws_secret_access_key => $aws_secret,
+              ( $aws_host ? ( host => $aws_host ) : () ),
+              authorization_method => 'Net::Amazon::S3::Signature::V4',
+            });
+          my $bucket = $s3->bucket($aws_bucket);
+          if ( !$bucket ) {
+            Error("S3 bucket $bucket not found.");
+            die;
+          }
+
+          my $event_path = $subpath.$self->RelativePath();
+          if ( 0 ) { # Not neccessary
+            Debug("Making directory $event_path/");
+            if ( !$bucket->add_key($event_path.'/', '') ) {
+              Warning("Unable to add key for $event_path/ :". $s3->err . ': '. $s3->errstr());
+            }
+          }
+
+          my @files = glob("$OldPath/*");
+          Debug("Files to move @files");
+          foreach my $file ( @files ) {
+            next if $file =~ /^\./;
+            ( $file ) = ( $file =~ /^(.*)$/ ); # De-taint
+            my $starttime = [gettimeofday];
+            Debug("Moving file $file to $NewPath");
+            my $size = -s $file;
+            if ( ! $size ) {
+              Info('Not moving file with 0 size');
+            }
+            if ( 0 ) {
+              my $file_contents = File::Slurp::read_file($file);
+              if ( ! $file_contents ) {
+                die 'Loaded empty file, but it had a size. Giving up';
+              }
+
+              my $filename = $event_path.'/'.File::Basename::basename($file);
+              if ( ! $bucket->add_key($filename, $file_contents) ) {
+                die "Unable to add key for $filename : ".$s3->err . ': ' . $s3->errstr;
+              }
+            } else {
+              my $filename = $event_path.'/'.File::Basename::basename($file);
+              if ( ! $bucket->add_key_filename($filename, $file) ) {
+                die "Unable to add key for $filename " . $s3->err . ': '. $s3->errstr;
+              }
+            }
+
+            my $duration = tv_interval($starttime);
+            Debug('PUT to S3 ' . Number::Bytes::Human::format_bytes($size) . " in $duration seconds = " . Number::Bytes::Human::format_bytes($duration?$size/$duration:$size) . '/sec');
+          } # end foreach file.
+
+          $moved = 1;
+        };
+        Error($@) if $@;
+      } else {
+        Error("Unable to parse S3 Url into it's component parts.");
       }
-
-      my $event_path = 'events/'.$self->RelativePath();
-Info("Making dir ectory $event_path/");
-      if ( ! $bucket->add_key( $event_path.'/','' ) ) {
-        die "Unable to add key for $event_path/";
-      }
-
-      my @files = glob("$OldPath/*");
-Debug("Files to move @files");
-      for my $file (@files) {
-        next if $file =~ /^\./;
-        ( $file ) = ( $file =~ /^(.*)$/ ); # De-taint
-         my $starttime = time;
-        Debug("Moving file $file to $NewPath");
-        my $size = -s $file;
-        if ( ! $size ) {
-          Info('Not moving file with 0 size');
-        }
-        my $file_contents = File::Slurp::read_file($file);
-        if ( ! $file_contents ) {
-          die 'Loaded empty file, but it had a size. Giving up';
-        }
-
-        my $filename = $event_path.'/'.File::Basename::basename($file);
-        if ( ! $bucket->add_key( $filename, $file_contents ) ) {
-          die "Unable to add key for $filename";
-        }
-        my $duration = time - $starttime;
-        Debug('PUT to S3 ' . Number::Bytes::Human::format_bytes($size) . " in $duration seconds = " . Number::Bytes::Human::format_bytes($duration?$size/$duration:$size) . '/sec');
-      } # end foreach file.
-
-      $moved = 1;
-    };
-    Error($@) if $@;
-    die $@ if $@;
+      #die $@ if $@;
+    } # end if Url
   } # end if s3
 
   my $error = '';
-  if ( ! $moved ) {
-    File::Path::make_path( $NewPath, {error => \my $err} );
+  if ( !$moved ) {
+    File::Path::make_path($NewPath, {error => \my $err});
     if ( @$err ) {
       for my $diag (@$err) {
         my ($file, $message) = %$diag;
         next if $message eq 'File exists';
-        if ($file eq '') {
+        if ( $file eq '' ) {
           $error .= "general error: $message\n";
         } else {
           $error .= "problem making $file: $message\n";
@@ -655,21 +709,21 @@ Debug("Files to move @files");
     my @files = glob("$OldPath/*");
     if ( ! @files ) {
       $ZoneMinder::Database::dbh->commit();
-      return "No files to move.";
+      return 'No files to move.';
     }
 
     for my $file (@files) {
       next if $file =~ /^\./;
       ( $file ) = ( $file =~ /^(.*)$/ ); # De-taint
-      my $starttime = time;
+      my $starttime = [gettimeofday];
       Debug("Moving file $file to $NewPath");
       my $size = -s $file;
       if ( ! File::Copy::copy( $file, $NewPath ) ) {
         $error .= "Copy failed: for $file to $NewPath: $!";
         last;
       }
-      my $duration = time - $starttime;
-      Debug("Copied " . Number::Bytes::Human::format_bytes($size) . " in $duration seconds = " . ($duration?Number::Bytes::Human::format_bytes($size/$duration):'inf') . "/sec");
+      my $duration = tv_interval($starttime);
+      Debug('Copied ' . Number::Bytes::Human::format_bytes($size) . " in $duration seconds = " . ($duration?Number::Bytes::Human::format_bytes($size/$duration):'inf') . '/sec');
     } # end foreach file.
   } # end if ! moved
 
@@ -677,22 +731,142 @@ Debug("Files to move @files");
     $ZoneMinder::Database::dbh->commit();
     return $error;
   }
+} # end sub CopyTo
+
+sub MoveTo {
+
+  my ( $self, $NewStorage ) = @_;
+  my $OldStorage = $self->Storage(undef);
+
+  my $error = $self->CopyTo($NewStorage);
+  return $error if $error;
 
   # Succeeded in copying all files, so we may now update the Event.
   $$self{StorageId} = $$NewStorage{Id};
-  $$self{Storage} = $NewStorage;
+  $self->Storage($NewStorage);
   $error .= $self->save();
   if ( $error ) {
     $ZoneMinder::Database::dbh->commit();
     return $error;
   }
-Debug("Committing");
   $ZoneMinder::Database::dbh->commit();
-  $self->delete_files( $OldStorage );
-Debug("Done deleting files, returning");
+  $self->delete_files($OldStorage);
   return $error;
 } # end sub MoveTo
 
+# Assumes $path is absolute
+#
+sub recover_timestamps {
+  my ( $Event, $path ) = @_;
+  $path = $Event->Path() if ! $path;
+
+  if ( !opendir(DIR, $path) ) {
+    Error("Can't open directory '$path': $!");
+    return;
+  }
+  my @contents = readdir(DIR);
+  Debug('Have ' . @contents . " files in $path");
+  closedir(DIR);
+
+  my @mp4_files = grep( /^\d+\-video\.mp4$/, @contents);
+  if ( @mp4_files ) {
+    $$Event{DefaultVideo} = $mp4_files[0];
+  }
+
+  my @analyse_jpgs = grep( /^\d+\-analyse\.jpg$/, @contents);
+  if ( @analyse_jpgs ) {
+    $$Event{Save_JPEGs} |= 2;
+  }
+
+  my @capture_jpgs = grep( /^\d+\-capture\.jpg$/, @contents);
+  if ( @capture_jpgs ) {
+    $$Event{Frames} = scalar @capture_jpgs;
+    $$Event{Save_JPEGs} |= 1;
+    # can get start and end times from stat'ing first and last jpg
+    @capture_jpgs = sort { $a cmp $b } @capture_jpgs;
+    my $first_file = "$path/$capture_jpgs[0]";
+    ( $first_file ) = $first_file =~ /^(.*)$/;
+    my $first_timestamp = (stat($first_file))[9];
+
+    my $last_file = "$path/$capture_jpgs[@capture_jpgs-1]";
+    ( $last_file ) = $last_file =~ /^(.*)$/;
+    my $last_timestamp = (stat($last_file))[9];
+
+    my $duration = $last_timestamp - $first_timestamp;
+    $Event->Length($duration);
+    $Event->StartTime( Date::Format::time2str('%Y-%m-%d %H:%M:%S', $first_timestamp) );
+    $Event->EndTime( Date::Format::time2str('%Y-%m-%d %H:%M:%S', $last_timestamp) );
+    Debug("From capture Jpegs have duration $duration = $last_timestamp - $first_timestamp : $$Event{StartTime} to $$Event{EndTime}");
+    $ZoneMinder::Database::dbh->begin_work();
+    foreach my $jpg ( @capture_jpgs ) {
+      my ( $id ) = $jpg =~ /^(\d+)\-capture\.jpg$/;
+
+      if ( ! ZoneMinder::Frame->find_one( EventId=>$$Event{Id}, FrameId=>$id ) ) {
+        my $file = "$path/$jpg";
+        ( $file ) = $file =~ /^(.*)$/;
+        my $timestamp = (stat($file))[9];
+        my $Frame = new ZoneMinder::Frame();
+        $Frame->save({
+            EventId=>$$Event{Id}, FrameId=>$id,
+            TimeStamp=>Date::Format::time2str('%Y-%m-%d %H:%M:%S',$timestamp),
+            Delta => $timestamp - $first_timestamp,
+            Type=>'Normal',
+            Score=>0,
+          });
+      }
+    }
+    $ZoneMinder::Database::dbh->commit();
+  } elsif ( @mp4_files ) {
+    my $file = "$path/$mp4_files[0]";
+    ( $file ) = $file =~ /^(.*)$/;
+
+    my $first_timestamp = (stat($file))[9];
+    my $output = `ffprobe $file 2>&1`;
+    my ($duration) = $output =~ /Duration: [:\.0-9]+/gm;
+    Debug("From mp4 have duration $duration, start: $first_timestamp");
+
+    my ( $h, $m, $s, $u );
+      if ( $duration =~ m/(\d+):(\d+):(\d+)\.(\d+)/ ) {
+        ( $h, $m, $s, $u ) = ($1, $2, $3, $4 );
+        Debug("( $h, $m, $s, $u ) from /^(\\d{2}):(\\d{2}):(\\d{2})\.(\\d+)/");
+      }
+    my $seconds = ($h*60*60)+($m*60)+$s;
+    $Event->Length($seconds.'.'.$u);
+    $Event->StartTime( Date::Format::time2str('%Y-%m-%d %H:%M:%S', $first_timestamp) );
+    $Event->EndTime( Date::Format::time2str('%Y-%m-%d %H:%M:%S', $first_timestamp+$seconds) );
+  }
+  if ( @mp4_files ) {
+    $Event->DefaultVideo($mp4_files[0]);
+  }
+}
+
+sub files {
+	my $self = shift;
+
+	if ( ! $$self{files} ) {
+		if ( ! opendir(DIR, $self->Path() ) ) {
+			Error("Can't open directory '$$self{Path}': $!");
+			return;
+		}
+		@{$$self{files}} = readdir(DIR);
+		Debug('Have ' . @{$$self{files}} . " files in $$self{Path}");
+		closedir(DIR);
+	}
+	return @{$$self{files}};
+}
+
+sub has_capture_jpegs {
+	@{$_[0]{capture_jpegs}} = grep(/^\d+\-capture\.jpg$/, $_[0]->files());
+	Debug("have " . @{$_[0]{capture_jpegs}} . " capture jpegs");
+	return @{$_[0]{capture_jpegs}} ? 1 : 0;
+}
+
+sub has_analyse_jpegs {
+	@{$_[0]{analyse_jpegs}} = grep(/^\d+\-analyse\.jpg$/, $_[0]->files());
+	Debug("have " . @{$_[0]{analyse_jpegs}} . " analyse jpegs");
+	return @{$_[0]{analyse_jpegs}} ? 1 : 0;
+}
+
 1;
 __END__
 
