@@ -32,8 +32,8 @@
 StreamBase::~StreamBase() {
 #if HAVE_LIBAVCODEC
   if ( vid_stream ) {
-		delete vid_stream;
-		vid_stream = NULL;
+    delete vid_stream;
+    vid_stream = NULL;
   }
 #endif
   closeComms();
@@ -44,7 +44,12 @@ bool StreamBase::loadMonitor(int monitor_id) {
     Error("Unable to load monitor id %d for streaming", monitor_id);
     return false;
   }
-  if ( ! monitor->connect() ) {
+  if ( monitor->GetFunction() == Monitor::NONE ) {
+    Error("Monitor %d has function NONE. Will not be able to connect to it.", monitor_id);
+    return false;
+  }
+
+  if ( !monitor->connect() ) {
     Error("Unable to connect to monitor id %d for streaming", monitor_id);
     return false;
   }
@@ -54,24 +59,34 @@ bool StreamBase::loadMonitor(int monitor_id) {
 
 bool StreamBase::checkInitialised() {
   if ( !monitor ) {
-    Fatal( "Cannot stream, not initialised" );
+    Fatal("Cannot stream, not initialised");
     return false;
   }
   return true;
 }
 
 void StreamBase::updateFrameRate(double fps) {
+  frame_mod = 1;
+  if ( (fps < 0) || !fps || isinf(fps) ) {
+    Debug(1, "Zero or negative fps %f in updateFrameRate. Setting frame_mod=1 and effective_fps=0.0", fps);
+    effective_fps = 0.0;
+    base_fps = 0.0;
+    return;
+  }
   base_fps = fps;
   effective_fps = (base_fps*abs(replay_rate))/ZM_RATE_BASE;
   frame_mod = 1;
-  Debug(3, "FPS:%.2f, MXFPS:%.2f, BFPS:%.2f, EFPS:%.2f, FM:%d", fps, maxfps, base_fps, effective_fps, frame_mod);
+  Debug(3, "FPS:%.2f, MaxFPS:%.2f, BaseFPS:%.2f, EffectiveFPS:%.2f, FrameMod:%d, replay_rate(%d)",
+      fps, maxfps, base_fps, effective_fps, frame_mod, replay_rate);
   // Min frame repeat?
-  while( effective_fps > maxfps ) {
+  // We want to keep the frame skip easy... problem is ... if effective = 31 and max = 30 then we end up with 15.5 fps.  
+  while ( effective_fps > maxfps ) {
     effective_fps /= 2.0;
     frame_mod *= 2;
-    Debug(3, "EffectiveFPS:%.2f, FrameMod:%d", effective_fps, frame_mod);
+    Debug(3, "Changing fps to be < max %.2f EffectiveFPS:%.2f, FrameMod:%d",
+        maxfps, effective_fps, frame_mod);
   }
-}
+} // void StreamBase::updateFrameRate(double fps)
 
 bool StreamBase::checkCommandQueue() {
   if ( sd >= 0 ) {
@@ -99,16 +114,7 @@ bool StreamBase::checkCommandQueue() {
   return false;
 }
 
-Image *StreamBase::prepareImage( Image *image ) {
-  static int last_scale = 0;
-  static int last_zoom = 0;
-  static int last_x = 0;
-  static int last_y = 0;
-
-  if ( !last_scale )
-    last_scale = scale;
-  if ( !last_zoom )
-    last_zoom = zoom;
+Image *StreamBase::prepareImage(Image *image) {
 
   // Do not bother to scale zoomed in images, just crop them and let the browser scale
   // Works in FF2 but breaks FF3 which doesn't like image sizes changing in mid stream.
@@ -118,50 +124,51 @@ Image *StreamBase::prepareImage( Image *image ) {
 
   int mag = (scale * zoom) / ZM_SCALE_BASE;
   int act_mag = optimisedScaling?(mag > ZM_SCALE_BASE?ZM_SCALE_BASE:mag):mag;
-  Debug( 3, "Scaling by %d, zooming by %d = magnifying by %d(%d)", scale, zoom, mag, act_mag );
 
   int last_mag = (last_scale * last_zoom) / ZM_SCALE_BASE;
   int last_act_mag = last_mag > ZM_SCALE_BASE?ZM_SCALE_BASE:last_mag;
-  Debug( 3, "Last scaling by %d, zooming by %d = magnifying by %d(%d)", last_scale, last_zoom, last_mag, last_act_mag );
-
   int base_image_width = image->Width(), base_image_height = image->Height();
-  Debug( 3, "Base image width = %d, height = %d", base_image_width, base_image_height );
-
   int virt_image_width = (base_image_width * mag) / ZM_SCALE_BASE, virt_image_height = (base_image_height * mag) / ZM_SCALE_BASE;
-  Debug( 3, "Virtual image width = %d, height = %d", virt_image_width, virt_image_height );
-
   int last_virt_image_width = (base_image_width * last_mag) / ZM_SCALE_BASE, last_virt_image_height = (base_image_height * last_mag) / ZM_SCALE_BASE;
-  Debug( 3, "Last virtual image width = %d, height = %d", last_virt_image_width, last_virt_image_height );
-
   int act_image_width = (base_image_width * act_mag ) / ZM_SCALE_BASE, act_image_height = (base_image_height * act_mag ) / ZM_SCALE_BASE;
-  Debug( 3, "Actual image width = %d, height = %d", act_image_width, act_image_height );
-
   int last_act_image_width = (base_image_width * last_act_mag ) / ZM_SCALE_BASE, last_act_image_height = (base_image_height * last_act_mag ) / ZM_SCALE_BASE;
-  Debug( 3, "Last actual image width = %d, height = %d", last_act_image_width, last_act_image_height );
-
   int disp_image_width = (image->Width() * scale) / ZM_SCALE_BASE, disp_image_height = (image->Height() * scale) / ZM_SCALE_BASE;
-  Debug( 3, "Display image width = %d, height = %d", disp_image_width, disp_image_height );
-
   int last_disp_image_width = (image->Width() * last_scale) / ZM_SCALE_BASE, last_disp_image_height = (image->Height() * last_scale) / ZM_SCALE_BASE;
-  Debug( 3, "Last display image width = %d, height = %d", last_disp_image_width, last_disp_image_height );
-
   int send_image_width = (disp_image_width * act_mag ) / mag, send_image_height = (disp_image_height * act_mag ) / mag;
-  Debug( 3, "Send image width = %d, height = %d", send_image_width, send_image_height );
-
   int last_send_image_width = (last_disp_image_width * last_act_mag ) / last_mag, last_send_image_height = (last_disp_image_height * last_act_mag ) / last_mag;
-  Debug( 3, "Last send image width = %d, height = %d", last_send_image_width, last_send_image_height );
 
-  if ( mag != ZM_SCALE_BASE ) {
-    if ( act_mag != ZM_SCALE_BASE ) {
-      Debug(3, "Magnifying by %d", mag);
-      if ( !image_copied ) {
-        static Image copy_image;
-        copy_image.Assign(*image);
-        image = &copy_image;
-        image_copied = true;
-      }
-      image->Scale(mag);
-    }
+  Debug(3,
+      "Scaling by %d, zooming by %d = magnifying by %d(%d)\n"
+      "Last scaling by %d, zooming by %d = magnifying by %d(%d)\n"
+      "Base image width = %d, height = %d\n"
+      "Virtual image width = %d, height = %d\n"
+      "Last virtual image width = %d, height = %d\n"
+      "Actual image width = %d, height = %d\n"
+      "Last actual image width = %d, height = %d\n"
+      "Display image width = %d, height = %d\n"
+      "Last display image width = %d, height = %d\n"
+      "Send image width = %d, height = %d\n"
+      "Last send image width = %d, height = %d\n",
+      scale, zoom, mag, act_mag,
+      last_scale, last_zoom, last_mag, last_act_mag,
+      base_image_width, base_image_height,
+      virt_image_width, virt_image_height,
+      last_virt_image_width, last_virt_image_height,
+      act_image_width, act_image_height,
+      last_act_image_width, last_act_image_height,
+      disp_image_width, disp_image_height,
+      last_disp_image_width, last_disp_image_height,
+      send_image_width, send_image_height,
+      last_send_image_width, last_send_image_height
+      );
+
+  if ( ( mag != ZM_SCALE_BASE ) && (act_mag != ZM_SCALE_BASE) ) {
+    Debug(3, "Magnifying by %d", mag);
+    static Image copy_image;
+    copy_image.Assign(*image);
+    image = &copy_image;
+    image_copied = true;
+    image->Scale(mag);
   }
 
   Debug(3, "Real image width = %d, height = %d", image->Width(), image->Height());
@@ -170,26 +177,22 @@ Image *StreamBase::prepareImage( Image *image ) {
     static Box last_crop;
 
     if ( mag != last_mag || x != last_x || y != last_y ) {
-      Debug( 3, "Got click at %d,%d x %d", x, y, mag );
-
-      //if ( !last_mag )
-        //last_mag = mag;
+      Debug(3, "Got click at %d,%d x %d", x, y, mag);
 
       if ( !(last_disp_image_width < last_virt_image_width || last_disp_image_height < last_virt_image_height) )
         last_crop = Box();
 
-      Debug( 3, "Recalculating crop" );
       // Recalculate crop parameters, as %ges
       int click_x = (last_crop.LoX() * 100 ) / last_act_image_width; // Initial crop offset from last image
       click_x += ( x * 100 ) / last_virt_image_width;
       int click_y = (last_crop.LoY() * 100 ) / last_act_image_height; // Initial crop offset from last image
       click_y += ( y * 100 ) / last_virt_image_height;
-      Debug( 3, "Got adjusted click at %d%%,%d%%", click_x, click_y );
+      Debug(3, "Got adjusted click at %d%%,%d%%", click_x, click_y);
 
       // Convert the click locations to the current image pixels
       click_x = ( click_x * act_image_width ) / 100;
       click_y = ( click_y * act_image_height ) / 100;
-      Debug( 3, "Got readjusted click at %d,%d", click_x, click_y );
+      Debug(3, "Got readjusted click at %d,%d", click_x, click_y);
 
       int lo_x = click_x - (send_image_width/2);
       if ( lo_x < 0 )
@@ -208,26 +211,29 @@ Image *StreamBase::prepareImage( Image *image ) {
         lo_y = hi_y - (send_image_height - 1);
       }
       last_crop = Box( lo_x, lo_y, hi_x, hi_y );
-    }
-    Debug( 3, "Cropping to %d,%d -> %d,%d", last_crop.LoX(), last_crop.LoY(), last_crop.HiX(), last_crop.HiY() );
+    }  // end if ( mag != last_mag || x != last_x || y != last_y )
+
+    Debug(3, "Cropping to %d,%d -> %d,%d", last_crop.LoX(), last_crop.LoY(), last_crop.HiX(), last_crop.HiY());
     if ( !image_copied ) {
       static Image copy_image;
-      copy_image.Assign( *image );
+      copy_image.Assign(*image);
       image = &copy_image;
       image_copied = true;
     }
-    image->Crop( last_crop );
-  }
+    image->Crop(last_crop);
+  }  // end if difference in image vs displayed dimensions
+
   last_scale = scale;
   last_zoom = zoom;
   last_x = x;
   last_y = y;
 
   return image;
-}
+}  // end Image *StreamBase::prepareImage(Image *image)
 
-bool StreamBase::sendTextFrame( const char *frame_text ) {
-  Debug(2, "Sending text frame '%s'", frame_text);
+bool StreamBase::sendTextFrame(const char *frame_text) {
+  Debug(2, "Sending %dx%d * %d text frame '%s'",
+      monitor->Width(), monitor->Height(), scale, frame_text);
 
   Image image(monitor->Width(), monitor->Height(), monitor->Colours(), monitor->SubpixelOrder());
   image.Annotate(frame_text, image.centreCoord(frame_text));
@@ -251,8 +257,8 @@ bool StreamBase::sendTextFrame( const char *frame_text ) {
 
     image.EncodeJpeg(buffer, &n_bytes);
 
-    fputs("--ZoneMinderFrame\r\nContent-Type: image/jpeg\r\n\r\n", stdout);
-    fprintf(stdout, "Content-Length: %d\r\n", n_bytes);
+    fputs("--ZoneMinderFrame\r\nContent-Type: image/jpeg\r\n", stdout);
+    fprintf(stdout, "Content-Length: %d\r\n\r\n", n_bytes);
     if ( fwrite(buffer, n_bytes, 1, stdout) != 1 ) {
       Error("Unable to send stream text frame: %s", strerror(errno));
       return false;
@@ -274,7 +280,7 @@ void StreamBase::openComms() {
       }
     }
 
-		unsigned int length = snprintf(
+    unsigned int length = snprintf(
         sock_path_lock,
         sizeof(sock_path_lock),
         "%s/zms-%06d.lock",
@@ -286,11 +292,23 @@ void StreamBase::openComms() {
     }
     Debug(1, "Trying to open the lock on %s", sock_path_lock);
 
+    // Under systemd, we get chrooted to something like /tmp/systemd-apache-blh/ so the dir may not exist.
+    if ( mkdir(staticConfig.PATH_SOCKS.c_str(), 0755) ) {
+      if ( errno != EEXIST ) {
+        Error("Can't mkdir %s: %s", staticConfig.PATH_SOCKS.c_str(), strerror(errno));
+        return;
+      } else {
+        Debug(3, "SOCKS dir %s already exists", staticConfig.PATH_SOCKS.c_str() );
+      }
+    } else {
+      Debug(3, "Success making SOCKS dir %s", staticConfig.PATH_SOCKS.c_str() );
+    }
+
     lock_fd = open(sock_path_lock, O_CREAT|O_WRONLY, S_IRUSR | S_IWUSR);
     if ( lock_fd <= 0 ) {
       Error("Unable to open sock lock file %s: %s", sock_path_lock, strerror(errno));
       lock_fd = 0;
-		} else if ( flock(lock_fd, LOCK_EX) != 0 ) {
+    } else if ( flock(lock_fd, LOCK_EX) != 0 ) {
       Error("Unable to lock sock lock file %s: %s", sock_path_lock, strerror(errno));
       close(lock_fd);
       lock_fd = 0;
@@ -301,8 +319,8 @@ void StreamBase::openComms() {
     sd = socket(AF_UNIX, SOCK_DGRAM, 0);
     if ( sd < 0 ) {
       Fatal("Can't create socket: %s", strerror(errno));
-		} else {
-			Debug(1, "Have socket %d", sd);
+    } else {
+      Debug(3, "Have socket %d", sd);
     }
 
     length = snprintf(
@@ -324,18 +342,18 @@ void StreamBase::openComms() {
 
     strncpy(loc_addr.sun_path, loc_sock_path, sizeof(loc_addr.sun_path));
     loc_addr.sun_family = AF_UNIX;
-		Debug(3, "Binding to %s", loc_sock_path);
+    Debug(3, "Binding to %s", loc_sock_path);
     if ( ::bind(sd, (struct sockaddr *)&loc_addr, strlen(loc_addr.sun_path)+sizeof(loc_addr.sun_family)+1) < 0 ) {
       Fatal("Can't bind: %s", strerror(errno));
     }
 
     snprintf(rem_sock_path, sizeof(rem_sock_path), "%s/zms-%06dw.sock", staticConfig.PATH_SOCKS.c_str(), connkey);
-    strncpy(rem_addr.sun_path, rem_sock_path, sizeof(rem_addr.sun_path)-1);
+    strncpy(rem_addr.sun_path, rem_sock_path, sizeof(rem_addr.sun_path));
     rem_addr.sun_family = AF_UNIX;
 
     gettimeofday(&last_comm_update, NULL);
   } // end if connKey > 0
-	Debug(2, "comms open");
+  Debug(3, "comms open at %s", loc_sock_path);
 } // end void StreamBase::openComms()
 
 void StreamBase::closeComms() {
